@@ -4,6 +4,8 @@ Contains interface (MultiDomainBasicAuth) and associated glue code for
 providing credentials in the context of network requests.
 """
 
+from __future__ import annotations
+
 import logging
 import os
 import shutil
@@ -12,13 +14,12 @@ import sysconfig
 import typing
 import urllib.parse
 from abc import ABC, abstractmethod
-from functools import lru_cache
+from functools import cache
 from os.path import commonprefix
 from pathlib import Path
-from typing import Any, Dict, List, NamedTuple, Optional, Tuple
+from typing import Any, NamedTuple
 
 from requests.auth import AuthBase, HTTPBasicAuth
-from requests.models import Request, Response
 from requests.utils import get_netrc_auth
 
 from pip._internal.utils.logging import getLogger
@@ -31,6 +32,10 @@ from pip._internal.utils.misc import (
 )
 from pip._internal.vcs.versioncontrol import AuthInfo
 
+if typing.TYPE_CHECKING:
+    from requests import PreparedRequest
+    from requests.models import Response
+
 logger = getLogger(__name__)
 
 KEYRING_DISABLED = False
@@ -48,9 +53,7 @@ class KeyRingBaseProvider(ABC):
     has_keyring: bool
 
     @abstractmethod
-    def get_auth_info(
-        self, url: str, username: Optional[str]
-    ) -> Optional[AuthInfo]: ...
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None: ...
 
     @abstractmethod
     def save_auth_info(self, url: str, username: str, password: str) -> None: ...
@@ -61,7 +64,7 @@ class KeyRingNullProvider(KeyRingBaseProvider):
 
     has_keyring = False
 
-    def get_auth_info(self, url: str, username: Optional[str]) -> Optional[AuthInfo]:
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None:
         return None
 
     def save_auth_info(self, url: str, username: str, password: str) -> None:
@@ -78,7 +81,7 @@ class KeyRingPythonProvider(KeyRingBaseProvider):
 
         self.keyring = keyring
 
-    def get_auth_info(self, url: str, username: Optional[str]) -> Optional[AuthInfo]:
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None:
         # Support keyring's get_credential interface which supports getting
         # credentials without a username. This is only available for
         # keyring>=15.2.0.
@@ -114,7 +117,7 @@ class KeyRingCliProvider(KeyRingBaseProvider):
     def __init__(self, cmd: str) -> None:
         self.keyring = cmd
 
-    def get_auth_info(self, url: str, username: Optional[str]) -> Optional[AuthInfo]:
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None:
         # This is the default implementation of keyring.get_credential
         # https://github.com/jaraco/keyring/blob/97689324abcf01bd1793d49063e7ca01e03d7d07/keyring/backend.py#L134-L139
         if username is not None:
@@ -126,7 +129,7 @@ class KeyRingCliProvider(KeyRingBaseProvider):
     def save_auth_info(self, url: str, username: str, password: str) -> None:
         return self._set_password(url, username, password)
 
-    def _get_password(self, service_name: str, username: str) -> Optional[str]:
+    def _get_password(self, service_name: str, username: str) -> str | None:
         """Mirror the implementation of keyring.get_password using cli"""
         if self.keyring is None:
             return None
@@ -159,7 +162,7 @@ class KeyRingCliProvider(KeyRingBaseProvider):
         return None
 
 
-@lru_cache(maxsize=None)
+@cache
 def get_keyring_provider(provider: str) -> KeyRingBaseProvider:
     logger.verbose("Keyring provider requested: %s", provider)
 
@@ -225,19 +228,19 @@ class MultiDomainBasicAuth(AuthBase):
     def __init__(
         self,
         prompting: bool = True,
-        index_urls: Optional[List[str]] = None,
+        index_urls: list[str] | None = None,
         keyring_provider: str = "auto",
     ) -> None:
         self.prompting = prompting
         self.index_urls = index_urls
-        self.keyring_provider = keyring_provider  # type: ignore[assignment]
-        self.passwords: Dict[str, AuthInfo] = {}
+        self.keyring_provider = keyring_provider
+        self.passwords: dict[str, AuthInfo] = {}
         # When the user is prompted to enter credentials and keyring is
         # available, we will offer to save them. If the user accepts,
         # this value is set to the credentials they entered. After the
         # request authenticates, the caller should call
         # ``save_credentials`` to save these.
-        self._credentials_to_save: Optional[Credentials] = None
+        self._credentials_to_save: Credentials | None = None
 
     @property
     def keyring_provider(self) -> KeyRingBaseProvider:
@@ -260,9 +263,9 @@ class MultiDomainBasicAuth(AuthBase):
 
     def _get_keyring_auth(
         self,
-        url: Optional[str],
-        username: Optional[str],
-    ) -> Optional[AuthInfo]:
+        url: str | None,
+        username: str | None,
+    ) -> AuthInfo | None:
         """Return the tuple auth for a given url from keyring."""
         # Do nothing if no url was provided
         if not url:
@@ -284,7 +287,7 @@ class MultiDomainBasicAuth(AuthBase):
             get_keyring_provider.cache_clear()
             return None
 
-    def _get_index_url(self, url: str) -> Optional[str]:
+    def _get_index_url(self, url: str) -> str | None:
         """Return the original index URL matching the requested URL.
 
         Cached or dynamically generated credentials may work against
@@ -391,7 +394,7 @@ class MultiDomainBasicAuth(AuthBase):
 
     def _get_url_and_credentials(
         self, original_url: str
-    ) -> Tuple[str, Optional[str], Optional[str]]:
+    ) -> tuple[str, str | None, str | None]:
         """Return the credentials to use for the provided URL.
 
         If allowed, netrc and keyring may be used to obtain the
@@ -437,8 +440,9 @@ class MultiDomainBasicAuth(AuthBase):
 
         return url, username, password
 
-    def __call__(self, req: Request) -> Request:
+    def __call__(self, req: PreparedRequest) -> PreparedRequest:
         # Get credentials for this request
+        assert req.url is not None
         url, username, password = self._get_url_and_credentials(req.url)
 
         # Set the url of the request to the url without any credentials
@@ -454,9 +458,7 @@ class MultiDomainBasicAuth(AuthBase):
         return req
 
     # Factored out to allow for easy patching in tests
-    def _prompt_for_password(
-        self, netloc: str
-    ) -> Tuple[Optional[str], Optional[str], bool]:
+    def _prompt_for_password(self, netloc: str) -> tuple[str | None, str | None, bool]:
         username = ask_input(f"User for {netloc}: ") if self.prompting else None
         if not username:
             return None, None, False
